@@ -375,24 +375,28 @@ def isPrime(N, false_positive_prob=1e-6, randfunc=None):
 import struct
 
 def long_to_bytes(n, blocksize=0):
-    """Convert an integer to a byte string.
+    """Convert a positive integer to a byte string using big endian encoding.
 
-    In Python 3.2+, use the native method instead::
-
-        >>> n.to_bytes(blocksize, 'big')
-
-    For instance::
-
-        >>> n = 80
-        >>> n.to_bytes(2, 'big')
-        b'\x00P'
-
-    If the optional :data:`blocksize` is provided and greater than zero,
-    the byte string is padded with binary zeros (on the front) so that
-    the total length of the output is a multiple of blocksize.
-
-    If :data:`blocksize` is zero or not provided, the byte string will
+    If :data:`blocksize` is absent or zero, the byte string will
     be of minimal length.
+
+    Otherwise, the length of the byte string is guaranteed to be a multiple
+    of :data:`blocksize`. If necessary, zeroes (``\\x00``) are added at the left.
+
+    .. note::
+        In Python 3, if you are sure that :data:`n` can fit into
+        :data:`blocksize` bytes, you can simply use the native method instead::
+
+            >>> n.to_bytes(blocksize, 'big')
+
+        For instance::
+
+            >>> n = 80
+            >>> n.to_bytes(2, 'big')
+            b'\\x00P'
+
+        However, and unlike this ``long_to_bytes()`` function,
+        an ``OverflowError`` exception is raised if :data:`n` does not fit.
     """
 
     if n < 0 or blocksize < 0:
@@ -401,32 +405,41 @@ def long_to_bytes(n, blocksize=0):
     result = []
     pack = struct.pack
 
-    while blocksize >= 8:
+    # Fill the first block independently from the value of n
+    bsr = blocksize
+    while bsr >= 8:
         result.insert(0, pack('>Q', n & 0xFFFFFFFFFFFFFFFF))
         n = n >> 64
-        blocksize -= 8
+        bsr -= 8
 
-    while blocksize >= 4:
+    while bsr >= 4:
         result.insert(0, pack('>I', n & 0xFFFFFFFF))
         n = n >> 32
-        blocksize -= 4
+        bsr -= 4
 
-    while blocksize > 0:
+    while bsr > 0:
         result.insert(0, pack('>B', n & 0xFF))
         n = n >> 8
-        blocksize -= 1
+        bsr -= 1
 
     if n == 0:
         if len(result) == 0:
-            result = [ b'\x00' ]
+            bresult = b'\x00'
+        else:
+            bresult = b''.join(result)
     else:
-        # The encoded number may exceed the block size
+        # The encoded number exceeds the block size
         while n > 0:
             result.insert(0, pack('>Q', n & 0xFFFFFFFFFFFFFFFF))
             n = n >> 64
         result[0] = result[0].lstrip(b'\x00')
+        bresult = b''.join(result)
+        # bresult has minimum length here
+        if blocksize > 0:
+            target_len = ((len(bresult) - 1) // blocksize + 1) * blocksize
+            bresult = b'\x00' * (target_len - len(bresult)) + bresult
 
-    return b''.join(result)
+    return bresult
 
 
 def bytes_to_long(s):
